"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Execution = void 0;
/**
 * Template execution, holds contexts, extra helpers, data.
 */
class Execution {
    /**
     * Template execution, holds contexts, extra helpers, data.
     *
     * @param {number} startMillis Timestamp of the template execution start
     * @param {object[]} contexts Contexts from which bigodon path expressions will evaluate
     * @param {Map<string, Function>} extraHelpers Extra helpers that can be called other than default bigodon helpers
     * @param {object?} data Data that cannot be accessed from the template but can be accessed and modified from helpers
     * @param {number} maxExecutionMillis Maximum milliseconds allowed for the template execution
     */
    constructor(startMillis, contexts, extraHelpers, data, maxExecutionMillis) {
        this.startMillis = startMillis;
        this.contexts = contexts;
        this.extraHelpers = extraHelpers;
        this.data = data;
        this.maxExecutionMillis = maxExecutionMillis;
    }
    /**
     * Current context to be used by path expressions.
     *
     * @return {object} Current context to be used by path expressions.
     */
    get context() {
        return this.contexts[this.contexts.length - 1];
    }
    /**
     * Milliseconds since the template execution started.
     * @return {number} Milliseconds since the template execution started.
     */
    get elapsedMillis() {
        return Date.now() - this.startMillis;
    }
    /**
     * Creates Execution from current one with new child context.
     * Used to change context allowing for $parent and $root access of previous contexts.
     *
     * @param {object} context New child context to be used by path expressions
     * @return {Execution} Execution with the added context
     */
    withChildContext(context) {
        return new Execution(this.startMillis, [...this.contexts, context], this.extraHelpers, this.data, this.maxExecutionMillis);
    }
    /**
     * Creates an execution from context, helpers and options.
     *
     * @param {object} context Context from which bigodon path expressions will evaluate
     * @param {Map<string, Function>?} extraHelpers Extra helpers that can be called other than default bigodon helpers
     * @param {BigodonOptions} options Options for the current execution only
     * @return {Execution}
     */
    static of(context, extraHelpers = new Map(), options = {}) {
        return new Execution(Date.now(), [context], extraHelpers, options.data, options.maxExecutionMillis || Infinity);
    }
}
exports.Execution = Execution;
